package com.bwie.auth.controller;

import com.bwie.auth.service.AuthService;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.LoginRequest;
import com.bwie.common.domain.request.PhoneLoginRequest;
import com.bwie.common.domain.request.UserReq;
import com.bwie.common.domain.request.UserRequest;
import com.bwie.common.domain.response.system.UserResponse;
import com.bwie.common.result.Result;
import org.springframework.web.bind.annotation.*;


@RestController
public class AuthController {


    private final AuthService authService;

    public AuthController(AuthService authService) {
        this.authService = authService;
    }

    /**
     * 登录
     * @param loginRequest
     * @return
     */
    @PostMapping("/login")
    public Result<UserResponse> login(@RequestBody LoginRequest loginRequest){
        return Result.success(
                authService.login(loginRequest)
        );
    }

    /**
     * 注册
     * @param userReq
     * @return
     */
    @PostMapping("/reg")
    public Result regUser(@RequestBody UserReq userReq){
        return authService.reg(userReq);
    }

    /**
     * 发送验证码
     * @param userTel
     * @return
     */
    @GetMapping("/sendCode/{userTel}")
    public Result sencCode(@PathVariable("userTel") String userTel){
        return authService.sendCode(userTel);
    }

    /**
     * 手机号登录
     * @param userTel
     * @return
     */
    @PostMapping("/sendCode/{userTel}")
    public Result sendCode(@PathVariable String userTel){

        Result result = authService.sencCode(userTel);

        return result;

    }

    /**
     * 获取验证码
     * @param phoneLoginRequest
     * @return
     */
    @PostMapping("/phoneLogin")
    public Result phoneLogin1(@RequestBody PhoneLoginRequest phoneLoginRequest){
        Result result = authService.phoneLogin(phoneLoginRequest);

        return Result.success(result);
    }

    /**
     *  获取用户信息
     * @return
     */
    @GetMapping("/info")
    public Result<User> getInfo(){
        return Result.success(
                authService.getInfo()
        );
    }

}
