package com.bwie.es.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bwie.common.domain.House;
import com.bwie.common.domain.request.ManagementRequest;
import com.bwie.common.remote.RemoteHouseService;
import com.bwie.common.result.Result;
import com.bwie.es.service.EsListService;
import org.elasticsearch.action.admin.indices.delete.DeleteIndexRequest;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.bulk.BulkResponse;
import org.elasticsearch.action.get.GetRequest;
import org.elasticsearch.action.get.GetResponse;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.support.master.AcknowledgedResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.xcontent.XContentType;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

@Service
public class EsListServiceImpl implements EsListService {


    private final RestHighLevelClient client;

    private final RemoteHouseService remoteHouseService;
    public EsListServiceImpl(RestHighLevelClient client, RemoteHouseService remoteHouseService) {
        this.client = client;
        this.remoteHouseService = remoteHouseService;
    }
    @Override
    public void managementSync(List<House> list){
        try {
            BulkRequest bulkRequest = new BulkRequest();
            list.forEach(houseEsRequest -> {
                bulkRequest.add(
                        new IndexRequest("houes")
                                .id(houseEsRequest.getId().toString())
                                .source(JSONObject.toJSONString(houseEsRequest), XContentType.JSON)
                );
            });
            BulkResponse bulk = client.bulk(bulkRequest, RequestOptions.DEFAULT);
            Assert.isTrue(!bulk.hasFailures(),"添加失败");
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
    @Override
    public Result managementList(ManagementRequest managementRequest){


        ArrayList<House> houses = new ArrayList<>();
        SearchRequest searchRequest = new SearchRequest("houes");
        SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        sourceBuilder.from((managementRequest.getPageNum()-1)*managementRequest.getPageSize());
        sourceBuilder.size(managementRequest.getPageSize());
//        if(managementRequest.){
//
//        }
        sourceBuilder.query(boolQueryBuilder);
        searchRequest.source(sourceBuilder);
        try {
            SearchResponse search = client.search(searchRequest, RequestOptions.DEFAULT);
            SearchHits hits = search.getHits();
            for (SearchHit hit : hits.getHits()) {
                String sourceAsString = hit.getSourceAsString();
                House house = JSON.parseObject(sourceAsString, House.class);
                houses.add(house);
            }
        }catch (IOException e) {
            throw new RuntimeException(e);
        }
        return Result.success(houses);
    }


    public GetResponse getDocumentById(String documentId) throws IOException {
        GetRequest getRequest = new GetRequest("houes", documentId);
        return client.get(getRequest, RequestOptions.DEFAULT);
    }


    /**
     * 逻辑删除内容
     */
    @Override
    public House printDocument(String id) throws IOException {
        House house = new House();
        GetResponse response = getDocumentById(id);
        if (response.isExists()) {
            String sourceAsString = response.getSourceAsString();
            house = JSON.parseObject(sourceAsString, House.class);
            System.out.println("这是个啥内容:::::"+response.getSourceAsString()); // 输出JSON格式的完整文档内容
            DeleteIndexRequest request = new DeleteIndexRequest(id);
            AcknowledgedResponse delete = client.indices().delete(request, RequestOptions.DEFAULT);
        } else {
            System.out.println("Document not found with ID: " + id);
        }
        return house;
    }





















}
