package com.bwie.es.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bwie.common.domain.request.HouseEsRequest;
import com.bwie.common.domain.request.HouseFindRequest;
import com.bwie.common.result.PageResult;
import com.bwie.common.utils.StringUtils;
import com.bwie.es.service.EsService;
import com.google.gson.JsonObject;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.bulk.BulkResponse;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;

import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.xcontent.XContentType;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

@Service
public class EsServiceImpl implements EsService {

    private final RestHighLevelClient client;

    public EsServiceImpl(RestHighLevelClient client) {
        this.client = client;
    }

    @Override
    public void syncHouse(List<HouseEsRequest> list) {
        try {
            BulkRequest bulkRequest = new BulkRequest();
            list.forEach(houseEsRequest -> {
                bulkRequest.add(
                        new IndexRequest("house")
                                .id(houseEsRequest.getId().toString())
                                .source(JSONObject.toJSONString(houseEsRequest), XContentType.JSON)
                );
            });
            BulkResponse bulk = client.bulk(bulkRequest, RequestOptions.DEFAULT);
            Assert.isTrue(!bulk.hasFailures(),"添加失败");
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public PageResult<HouseEsRequest> getEsList(HouseFindRequest req) {
        SearchRequest searchRequest = new SearchRequest("house");
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        BoolQueryBuilder boolQueryBuilder = new BoolQueryBuilder();
        ArrayList<HouseEsRequest> list = new ArrayList<>();
        Long total = 0L;

        String start = "";
        String end = "";
//        private Long roomTypeId; // 字典详情 id 商户 写字楼小区
//
//        private Long addrId;  //区域id
//        private String housePrice;  //价格
//
//        private String roomType;  //户型
//        private String orientation;  //朝向
//        private Integer verify;  //是否验真【0-已验真 1-未验真】

        try {
            if (req.getRoomTypeId() != null){
                boolQueryBuilder.must(QueryBuilders.termQuery("dictionaryTypeId",req.getRoomTypeId()));
            }
            if (null != req.getAddrId()){
                boolQueryBuilder.must(QueryBuilders.termQuery("addrId",req.getAddrId()));
            }
            if (StringUtils.isNotBlank(req.getHousePrice())){
                boolean starts = req.getHousePrice().startsWith("_");
                boolean ends = req.getHousePrice().endsWith("_");
                if (starts){
                    String[] split = req.getHousePrice().split("_");
                    end = split[1];
                }
                if (ends){
                    String[] split = req.getHousePrice().split("_");
                    start = split[0];
                }

                if (starts==false && ends==false){
                    String[] split = req.getHousePrice().split("_");
                    start = split[0];
                    end = split[1];
                }

                if (StringUtils.isNotBlank(start)){
                    boolQueryBuilder.must(QueryBuilders.rangeQuery("housePrice").lte(req.getHousePrice()));
                }
                if (StringUtils.isNotBlank(end)){
                    boolQueryBuilder.must(QueryBuilders.rangeQuery("housePrice").gte(req.getHousePrice()));
                }
            }

            //        private String roomType;  //户型
//        private String orientation;  //朝向
//        private Integer verify;  //是否验真【0-已验真 1-未验真】
            if (StringUtils.isNotBlank(req.getRoomType())){
                boolQueryBuilder.must(QueryBuilders.matchQuery("roomType",req.getRoomType()));
            }
            if (StringUtils.isNotBlank(req.getOrientation())){
                boolQueryBuilder.must(QueryBuilders.termQuery("orientation",req.getOrientation()));
            }
            if (null != req.getVerify()){
                boolQueryBuilder.must(QueryBuilders.termQuery("verify",req.getVerify()));
            }

            searchSourceBuilder.query(boolQueryBuilder);

            searchSourceBuilder.from((req.getPageNum()-1)*req.getPageSize());
            searchSourceBuilder.size(req.getPageSize());

            SearchRequest source = searchRequest.source(searchSourceBuilder);

            SearchResponse search = client.search(source, RequestOptions.DEFAULT);

            total = search.getHits().getTotalHits().value;

            for (SearchHit hit : search.getHits().getHits()) {
                String sourceAsString = hit.getSourceAsString();
                HouseEsRequest houseEsRequest = JSONObject.parseObject(sourceAsString, HouseEsRequest.class);
                list.add(houseEsRequest);
            }

        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return PageResult.toPageResult(total,list);
    }
}
