package com.bwie.esask.seriver.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bwie.esask.feign.AskBuyRentFeign;
import com.bwie.esask.mapper.EsAskBuyMapper;
import com.bwie.esask.seriver.EsAskBuyService;
import com.bwie.common.domain.AskBuy;
import com.bwie.common.domain.request.AskBuyRequest;
import com.bwie.common.redis.RedisCache;
import com.bwie.common.result.Result;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.xcontent.XContentType;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * Es求购表
 */
@Service
public class EsAskBuyServiceImpl extends ServiceImpl<EsAskBuyMapper, AskBuy> implements EsAskBuyService {

    private final RedisCache redisCache;


    private final RestHighLevelClient client;

    private final AskBuyRentFeign askBuyRentFeign;

    public EsAskBuyServiceImpl(RedisCache redisCache,RestHighLevelClient client, AskBuyRentFeign askBuyRentFeign) {
        this.redisCache = redisCache;
        this.client = client;
        this.askBuyRentFeign = askBuyRentFeign;
    }

    @Override
    public void shouList(List<AskBuy> list){
        try {
            BulkRequest bulkRequest = new BulkRequest();
            list.forEach(askBuy -> {
                bulkRequest.add(
                  new IndexRequest("askbuy")
                          .id(JSON.toJSONString(askBuy.getAskBuyId()))
                          .source(JSON.toJSONString(askBuy),XContentType.JSON)
                );
            });
            client.bulk(bulkRequest, RequestOptions.DEFAULT);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
    @Override
    public Result shouBuyList(AskBuyRequest askBuyRequest){
        ArrayList<AskBuy> arrayList = new ArrayList<>();
        SearchRequest searchRequest = new SearchRequest("askbuy");
        SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        sourceBuilder.from((askBuyRequest.getPageNum()-1)*askBuyRequest.getPageSize());
        sourceBuilder.size(arrayList.size());
        sourceBuilder.query(boolQueryBuilder);
        searchRequest.source(sourceBuilder);
        try {
            SearchResponse search = client.search(searchRequest, RequestOptions.DEFAULT);
            SearchHits hits = search.getHits();
            if(hits!=null){
                for (SearchHit hit : hits.getHits()) {
                    String sourceAsString = hit.getSourceAsString();
                    AskBuy askBuy = JSON.parseObject(sourceAsString, AskBuy.class);
                    arrayList.add(askBuy);
                }

            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return Result.success(arrayList);
    }
}
