package com.bwie.home.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bwie.common.domain.DictionaryType;
import com.bwie.common.domain.House;
import com.bwie.common.domain.HouseImages;
import com.bwie.common.domain.HouseType;
import com.bwie.common.domain.request.HouseEsRequest;
import com.bwie.home.mapper.HouseMapper;
import com.bwie.home.service.DictionaryTypeService;
import com.bwie.home.service.HouseImagesService;
import com.bwie.home.service.HouseService;
import com.bwie.home.service.HouseTypeService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@Service
public class HouseServiceImpl extends ServiceImpl<HouseMapper, House>
        implements HouseService {

    private final HouseTypeService houseTypeService;
    private final DictionaryTypeService dictionaryTypeService;
    private final HouseImagesService houseImagesService;

    public HouseServiceImpl(HouseTypeService houseTypeService,
                            DictionaryTypeService dictionaryTypeService,
                            HouseImagesService houseImagesService) {
        this.houseTypeService = houseTypeService;
        this.dictionaryTypeService = dictionaryTypeService;
        this.houseImagesService = houseImagesService;
    }

    @Override
    public List<HouseEsRequest> getHouseEsList() {

        List<House> list = this.list();
        List<HouseEsRequest> esRequestList = list.stream().map(c -> {
            HouseType houseType = houseTypeService.getById(c.getHouseTypeId());
            DictionaryType dictionaryType = dictionaryTypeService.getById(c.getDictionaryTypeId());
            List<HouseImages> houseImagesList = houseImagesService.list(new LambdaQueryWrapper<HouseImages>()
                    .eq(HouseImages::getHouseId, c.getId()));
            List<String> collect = houseImagesList.stream().map(img -> {
                return img.getImageAddr();
            }).collect(Collectors.toList());
            HouseEsRequest houseEsRequest = new HouseEsRequest();
            BeanUtils.copyProperties(c, houseEsRequest);
            houseEsRequest.setHouseTypeName(houseType.getName());
            houseEsRequest.setDictionaryTypeName(dictionaryType.getDictionaryTypeLabel());
            houseEsRequest.setHouseImages(collect);
            return houseEsRequest;
        }).collect(Collectors.toList());

        return esRequestList;
    }
}
